import pygame
from pygame.locals import *

from modules import *

###GLOBALS
WIDTH = 640
HEIGHT = 480

def main():

    ###INIT PYGAME
    pygame.init()
    screen = pygame.display.set_mode((WIDTH, HEIGHT))
    pygame.display.set_caption("Pygame")

    ###MIXER
    pygame.mixer.init()
    pygame.mixer.music.load(os.path.join("data", "sound", "beat_loop.mp3"))
    pygame.mixer.music.play(-1)

    ###VARIABLES
    font_path = os.path.join("data", "fonts", "2DB.ttf")
    big_font = pygame.font.Font(font_path, 60)
    small_font = pygame.font.Font(font_path, 20)
    clock = pygame.time.Clock()
    menu_option = 0
    levels_unlocked = 0
    level_select = 1
    text1_posy, text2_posy = 480, 0
    global GAME_MODE, MENU, LEVEL_SELECT, ON_GAME, GAME_OVER

    ###INIT GROUPS
    #MAP.add(Map(screen))
    #PLAYER.add(Player((250, 390), screen, MAP.sprite))
    #HUD.add(Hud(PLAYER.sprite, screen))

    ###GAME LOOP
    while True:

        ###CLEAN THE SCREEN
        screen.fill((0, 0, 0))

        ###MENU MODE
        if GAME_MODE == MENU:

            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                        sys.exit()
                elif event.type == pygame.KEYDOWN:
                    if event.key == K_UP:
                        menu_option = 0
                    elif event.key == K_DOWN:
                        menu_option = 1
                    elif event.key == K_RETURN:
                        if not menu_option:
                            GAME_MODE = LEVEL_SELECT
                            menu_option = 0
                        else:
                            sys.exit()
                    elif event.key == K_ESCAPE:
                        sys.exit()

            ###DRAW
            screen.blit(big_font.render(">>", 1, (192, 192, 192)), (200, 150 + (75 * menu_option)))
            screen.blit(big_font.render("START", 1, (192, 192, 192)), (250, 150))
            screen.blit(big_font.render("QUIT", 1, (192, 192, 192)), (250, 230))

        ###LEVEL_SELECT MODE
        elif GAME_MODE == LEVEL_SELECT:

            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    sys.exit()
                elif event.type == pygame.KEYDOWN:
                    if event.key == K_UP:
                        if level_select > 1:
                            level_select -= 1
                    elif event.key == K_DOWN:
                        if level_select < levels_unlocked:
                            level_select += 1
                    elif event.key == K_RETURN:
                        ###START A NEW GAME
                        MAP.add(Map(screen, level_select))
                        PLAYER.add(Player((250, 390), screen, MAP.sprite))
                        HUD.add(Hud(PLAYER.sprite, screen, small_font))

                        GAME_MODE = ON_GAME
                        level_select = 1
                    elif event.key == K_ESCAPE:
                        GAME_MODE = MENU

            if not levels_unlocked:
                ###LOAD THE LEVELS UNLOCKED
                path = os.path.join("data", "user", "data.dat")
                shelf = shelve.open(path)
                try:
                    levels_unlocked = shelf["max"]
                except:
                    levels_unlocked = 1
                shelf.close()

            ###SHOW THE LEVELS AVAILABLE
            for n in range(levels_unlocked):
                text = small_font.render("LEVEL " + str(n + 1), 1, (192, 192, 192))
                screen.blit(text, (270, 50 + ((25 * n) + 1)))

            cursor = small_font.render(">>", 1, (192, 192, 192))
            screen.blit(cursor, (250, 50 + (25 * (level_select - 1))))

        ###ON_GAME MODE
        elif GAME_MODE == ON_GAME:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                        sys.exit()
                elif event.type == pygame.KEYDOWN:
                    if event.key == K_ESCAPE:
                        empty_groups()
                        GAME_MODE = MENU
                        levels_unlocked = 0

            update_groups()
            if PLAYER.sprite and PLAYER.sprite.lifes < 1 \
               or MAP.sprite and MAP.sprite.level > MAX_LEVELS:
                if not PLAYER.sprite.lifes: 
                    GAME_MODE = GAME_OVER
                else:
                    GAME_MODE = MENU
                empty_groups()
                text1_posy, text2_posy = 480, 0
                levels_unlocked = 0

        ###GAME_OVER MODE
        elif GAME_MODE == GAME_OVER:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                        sys.exit()
                elif event.type == pygame.KEYDOWN:
                    if event.key == K_RETURN or event.key == K_ESCAPE:
                        GAME_MODE = MENU

            if text1_posy > 150:
                text1_posy -= 10
            if text2_posy < 210:
                text2_posy += 7

            screen.blit(big_font.render("GAME", 1, (192, 192, 192)), (250, text1_posy))
            screen.blit(big_font.render("OVER", 1, (192, 192, 192)), (250, text2_posy))

        ###CONTROL FPS
        clock.tick(60)

        pygame.display.flip()
main()

