import pygame
from pygame.locals import *

import os
import shelve

import functions
from objects import *
from enemies import *
from globals import *

class Map(pygame.sprite.Sprite):

    def __init__(self, _screen, level = 1):

        pygame.sprite.Sprite.__init__(self)
        self.image = functions.load_image("tiles.png")
        self.screen = _screen
        self.map = []
        for n in range(300):
            self.map.append(0)
        self.level = level
        self.transition = False
        self.alpha = 255
        self.black = functions.load_image("t001.png")
        self.load_level(level)

    def __getitem__(self, key):

        return self.map[key]

    def load_level(self, level):

        path = os.path.join("data", "levels", "level" + str(level) + ".dat")
        f = open(path, "r")
        m = f.read()
        f.close()

        ###VARIOUS
        e = {"#" : Monster,
             "@" : Tank,
             "r" : 1,
             "l" : -1}

        nx = -1
        ny = 0
        index = -1
        for c in m:
            index += 1
            nx += 1
            try:
                ###START POINT
                if int(c) == 9:
                    self.map[index] = 9
                ###PILLS
                elif int(c) > 5:
                    OBJECTS.add(Pills(((32 * nx) + 8, 32 * ny), self.screen, (int(c) - 5) * -1))
                    self.map[index] = 0
                ###OTHERS
                else:
                    self.map[index] = int(c)
            except:
                try:
                    ###ENEMIES
                    ENEMIES.add(e[c]((32 * nx, 32 * ny), self.screen))
                except:
                    ###PORTALS
                    OBJECTS.add(Portal((32 * nx, 32 * ny), self.screen, e[c]))
                self.map[index] = 0

            if(nx >= 20):
                nx = 0
                ny += 1

    def load_next_level(self):

        self.level += 1
        if self.level < MAX_LEVELS + 1:
            self.load_level(self.level)
            self.save()
            self.transition = True

    def update(self):

        for y in range(15):
            for x in range(20):
                tile = self.map[x + 20 * y]
                if tile:
                    self.screen.blit(self.image, (32 * x, 32 * y), ((tile - 1) * 32, 0, 32, 32))

        ###TRANSITION BETWEEN LEVELS
        if self.transition:
            self.alpha -= 5
            if self.alpha <= 0:
                self.transition, self.alpha = False, 255
            else:
                self.black.set_alpha(self.alpha)
                self.screen.blit(self.black, (0, 0))

    def save(self):

        path = os.path.join("data", "user", "data.dat")
        shelf = shelve.open(path)
        try:
            if shelf["max"] < self.level:
                shelf["max"] = self.level
        except:
            shelf["max"] = self.level
        shelf.close()
