import pygame
from pygame.locals import *

import os
import shelve

import functions
from globals import *
from effects import Explosion
from objects import Portal


GRAVITY_FORCE = 0.8

class Player(pygame.sprite.Sprite):

    def __init__(self, pos, _screen, _map):

        pygame.sprite.Sprite.__init__(self)
        self.images = {
                        0 : functions.load_image("001.png", "colorkey"),
                        1 : functions.load_image("002.png", "colorkey"),
                        2 : functions.load_image("003.png", "alpha"),
                        3 : functions.load_image("004.png", "alpha")
                      }
        self.frames = None
        self.max_frames = None
        self.frame = None
        self.anim_delay = 5
        self.delay = None
        self.always_update = None
        self.visible = True

        self.rect = pygame.Rect(0, 0, 0, 0)
        self.rect.center = pos

        self.onGround = 0
        self.orientation = 1
        self.PLAYER_SPEED = None
        self.dirY = 0

        self.lifes = None
        self.respawn_delay = 0

        self.screen = _screen
        self.map = _map.map

        self.powers = None
        self.state = None

        self.start()

    def update(self):

        self.dirY += GRAVITY_FORCE

        f = 0

        keys = pygame.key.get_pressed()
        ###HORIZONTAL MOVEMENT
        if keys[K_RIGHT] or keys[K_LEFT]:

            y1 = self.rect.bottom / TILE_SIZE
            y2 = self.rect.top / TILE_SIZE
            y3 = self.rect.centery / TILE_SIZE

            if keys[K_RIGHT]:
                self.orientation = 1
                x1 = (self.rect.right + self.PLAYER_SPEED) / TILE_SIZE
                if x1 < 20:
                    if not self.map[x1 + y1 * 20] and not self.map[x1 + y2 * 20] \
                        and not self.map[x1 + y3 * 20]:
                            self.rect.move_ip(self.PLAYER_SPEED, 0)
                            f = 1
                else:
                    self.rect.move_ip(self.PLAYER_SPEED, 0)

            elif keys[K_LEFT]:
                self.orientation = -1
                x1 = (self.rect.left - self.PLAYER_SPEED) / TILE_SIZE
                if x1 > -1:
                    if not self.map[x1 + y1 * 20] and not self.map[x1 + y2 * 20] \
                        and not self.map[x1 + y3 * 20]:
                            self.rect.move_ip(-self.PLAYER_SPEED, 0)
                            f = 1
                else:
                    self.rect.move_ip(-self.PLAYER_SPEED, 0)

        ###VERTICAL MOVEMENT
        x1 = self.rect.left / TILE_SIZE
        x2 = self.rect.right / TILE_SIZE

        y1 = (self.rect.bottom + self.dirY) / TILE_SIZE
        y2 = (self.rect.top + self.dirY) / TILE_SIZE

        if self.map[x1 + int(y1) * 20] or self.map[x2 + int(y1) * 20]:
            self.dirY = 0
        elif self.map[x1 + int(y2) * 20] or self.map[x2 + int(y2) * 20] \
            or self.rect.top < 0:
                self.dirY = 1

        ###GRAVITY ACTION
        self.rect.centery += self.dirY

        ###JUMP
        if keys[K_SPACE]:
            if self.state == 2:
                self.dirY = -5
            elif self.dirY == 0:
                self.dirY = -11

        ###PUNCH
        if keys[K_LCTRL]:
            if self.state == 3:
                self.punch()

        ###POWERS
        if keys[K_0]:
            self.mutate(0)
        elif keys[K_1]:
            self.mutate(1)
        elif keys[K_2]:
            self.mutate(2)
        elif keys[K_3]:
            self.mutate(3)

        ###COLLISIONS
        self.collision()

        ###TIME FOR RESPAWN
        if self.respawn_delay:
            self.remove_life()

        ###NEXT LEVEL?
        if self.rect.left < 0 or self.rect.right > 640:
            ###FREE GROUPS
            ENEMIES.empty()
            OBJECTS.empty()
            ###LOAD NEXT LEVEL
            MAP.sprite.load_next_level()
            ###RESTART PLAYER
            self.start()

        ###DRAW THE PLAYER
        self.draw(f)

    def collision(self):

        ###OBJECTS COLLISIONS
        for e in OBJECTS:
            if e.rect.centerx - self.rect.centerx < 10 \
              and self.rect.centerx - e.rect.centerx < 10:
                if self.rect.colliderect(e.rect):
                    if isinstance(e, Portal):
                        e.teleport(self)
                        break
                    else:
                        self.powers.append(e.type + 1)
                        HUD.sprite.powers.append(e.type + 1)
                        e.kill()

        ###ENEMIES COLLISIONS
        if pygame.sprite.spritecollideany(self, ENEMIES):
            if not self.respawn_delay:
                self.remove_life(1)

    def mutate(self, p):

        if self.powers.count(p) and self.state != p:
            self.load_player_info(p)
            self.state = p
            EFFECTS.add(Explosion((self.rect.center), self.screen, 1))

    def load_player_info(self, type):

        ###LOAD THE FILE WITH THE INFO
        path = os.path.join("data", "info", "001.dat")
        shelf = shelve.open(path)
        data = shelf[str(type)]
        shelf.close()

        ###SET VALUES
        self.frames = data[0]
        self.max_frames = data[1]
        self.frame = data[1]
        self.delay = data[2]
        self.PLAYER_SPEED = data[3]
        self.always_update = data[4]

        self.rect.bottom -= 10
        self.rect.w = data[0][1][2]
        self.rect.h = data[0][1][3]

    def draw(self, f):

        self.anim_delay -= 1
        if not f and not self.always_update:
            self.frame = 1
        else:
            if self.anim_delay <= 0:
                self.frame += 1
                if self.frame > self.max_frames:
                    if not self.always_update:
                        self.frame = 2
                    else:
                        self.frame = 1
                self.anim_delay = self.delay

        if self.visible:
            self.screen.blit(self.images[self.state], self.rect, \
                self.frames[self.frame * self.orientation])

        #pygame.draw.rect(self.screen, (128, 0, 128), self.rect, 1)

    def punch(self):

        y = self.rect.top / TILE_SIZE
        if self.orientation > 0:
            x = (self.rect.left + TILE_SIZE) / TILE_SIZE
            if(self.map[x + y * 20] == 5):
                self.map[x + y * 20] = 0
                self.map[x + (y + 1) * 20] = 0
                EFFECTS.add(Explosion(((TILE_SIZE * x) + (TILE_SIZE / 2), \
                           (TILE_SIZE * y) + TILE_SIZE), self.screen, 2))
                return True

        elif self.orientation < 0:
            x = (self.rect.right - TILE_SIZE) / TILE_SIZE
            if(self.map[x + y * 20] == 5):
                self.map[x + y * 20] = 0
                self.map[x + (y + 1) * 20] = 0
                EFFECTS.add(Explosion(((TILE_SIZE * x) + (TILE_SIZE / 2), \
                           (TILE_SIZE * y) + TILE_SIZE), self.screen, 2))
                return True

        return False

    def remove_life(self, first = 0):

        if first:
            self.lifes -= 1
            if not self.lifes:
                pass
            else:
                self.respawn_delay = 150

        self.respawn_delay -= 1
        if self.respawn_delay:
            if not self.respawn_delay % 3:
                self.visible = False if self.visible else True
        else:
            self.visible = True

    def start(self):

        ###SET ALL TO DEFAULT
        self.load_player_info(0)

        self.powers = [0]
        self.state = 0
        self.lifes = 3

        if HUD.sprite:
            HUD.sprite.powers = []

        ###SET POS
        x, y = -1, 0
        for n in self.map:
            x += 1
            if n == 9:
                self.rect.center = (TILE_SIZE * x, TILE_SIZE * y)
                self.map[x + (y * 20)] = 0
                break
            if x >= 20:
                x = 0
                y += 1
